export declare namespace IOperator {
    type ILogic = '$less' | '$lessOrEqual' | '$greater' | '$greaterOrEqual' | '$equal' | '$diff';
    type IModifier = '$in' | '$notIn';
    type IOptions = ILogic | IModifier;
    type IDataSchemaKey = '$less:$lessOrEqual:$greater:$greaterOrEqual' | '$equal:$diff' | '$in:$notIn';
    type ILess = number;
    type IGreater = number;
    type IEqual = number | boolean | string;
    type In = string | [] | Record<string, any>;
    type Idle = unknown | any;
    type IBuilder = {
        result: boolean;
        method: IOptions;
        values: {
            a: any;
            b: any;
        };
    };
    type IMethodsLogic = {
        [k in ILogic]: <A>(a: A, b: A) => IBuilder;
    };
    type IMethodsModifier = {
        [k in IModifier]: <A, B>(a: A, b: B) => IBuilder;
    };
    type IMethods = IMethodsLogic & IMethodsModifier;
}
export declare namespace IDataType {
    type IOptions = 'string' | 'number' | 'bigint' | 'boolean' | 'symbol' | 'undefined' | 'object' | 'function';
    type ISchemaCheck = {
        checked: boolean;
        equals: boolean;
        types: Record<'a' | 'b', IOptions>;
    };
    type ISchemaOptions = {
        key: IOperator.IDataSchemaKey;
        types: IOptions[];
    };
    type IOperator = Record<IOperator.IDataSchemaKey, IOptions[]>;
}
export declare namespace IEvents {
    type IOptions = '$throw' | '$done';
    type ISchema = {
        condition: IOperator.IBuilder;
        event?: Function;
    };
    type IMethods = {
        $throw: (message: string) => Function;
        $when: (condition: IOperator.IBuilder, event?: Function) => ISchema;
    };
}
export declare namespace IEngine {
    type IRulesSchema<T> = Record<keyof T, IEvents.ISchema>;
    type IRules<T = any> = (metadata: T) => IRulesSchema<T>;
    type IFacts<T = any> = T;
    type IContextStatus = 'success' | 'failed';
    type IContext<R> = {
        [k in keyof R]: {
            status: IContextStatus;
            method: IOperator.IOptions;
            values: {
                expected: any;
                sended: any;
            };
            event?: Function;
        };
    };
}
